--==============================================================================
--miniSipServer Caller prepaid service script
--
--Copyright (C) MYVOIPAPP,Inc. All Rights Reserved.
--
--author: Gilson
--
--History:
--2011-03-05 Migrated from MSS core.      
--2011-08-30 Support one-time fee in prepaid rate
--2016-07-20 Migrated from Python script.
--==============================================================================

require "services"
require "utils.dba"
require "utils.date_time"

-- service states
local ECALLER_PREPAID_IDLE            = 0
local ECALLER_PREPAID_WAIT_ANS        = 1
local ECALLER_PREPAID_WAIT_DISC       = 2
local ECALLER_PREPAID_WAIT_ANN_END    = 3

-- default value
local DEFAULT_MATCH_TAG = "*"
local TIMER_WAIT_ANS = 120 -- 120 seconds

-- local FSM data
local subNbr=""
local balance=0
local rateId=0
local interval=0
local tariff=0
local onetimeFee=0
local fee=0
local allowDuration=0
local duration=0
local expireInd=0
local setupTime=0
local answerTime=0
local releaseTime=0

local rateInfo={interval=0, tariff=0, onetimeFee=0}


--*************************************
--
-- Common functions
--
--*************************************
local function _db_caller_prepaid_get_rateid(caller_nbr, called_nbr)
    local sqlStr = string.format("select nRateId from tbl_caller_prepaid where cSubscriber='%s' and cRouteNbr ='%s'", caller_nbr,called_nbr)
    DbPrepare()
    local result = DbQuery(sqlStr)
    if MSS_ERR_OK~=result or 0==DbGetRecordCnts() then
        DbEnd()
        return 0
    else
        DbFetchRow()
        result = DbGetRowIntVal(0)
        DbEnd()
        return result
    end
end

local function _db_caller_prepaid_get_rateid_for_any_sub(called_nbr)
    local sqlStr = string.format("select nRateId from tbl_caller_prepaid where cSubscriber='%s' and position(cRouteNbr in '%s')=1 order by length(cRouteNbr) desc limit 1", DEFAULT_MATCH_TAG, called_nbr)
    DbPrepare()
    local result = DbQuery(sqlStr)
    if MSS_ERR_OK~=result or 0==DbGetRecordCnts() then
        DbEnd()
        return 0
    else
        DbFetchRow()
        result = DbGetRowIntVal(0)
        DbEnd()
        return result
    end
end

local function db_caller_prepaid_get_rateid()
    local rate_id=0
    local sqlStr=string.format("select nRateId from tbl_caller_prepaid where cSubscriber='%s' and position(cRouteNbr in '%s')=1 order by length(cRouteNbr) desc limit 1", subNbr,fsm.calledNbr)
    
    DbPrepare()
    local result = DbQuery(sqlStr)
    if MSS_ERR_OK==result then
        DbFetchRow()
        rate_id = DbGetRowIntVal(0) 
    end
    DbEnd()
    
    -- check any route number for the subscrier
    if rate_id==0 then rate_id = _db_caller_prepaid_get_rateid(subNbr, DEFAULT_MATCH_TAG) end
    
    -- check any subscriber for this route number
    if rate_id==0 then rate_id = _db_caller_prepaid_get_rateid_for_any_sub(fsm.calledNbr) end
    
    -- check any subscriber for any route number
    if rate_id==0 then rate_id = _db_caller_prepaid_get_rateid(DEFAULT_MATCH_TAG, DEFAULT_MATCH_TAG) end
    
    return rate_id
end


local function db_caller_prepaid_get_rate_info()
    if rateId<=0 then return -1 end
    
    local sqlStr=string.format("select tariff_interval, tariff, onetime_fee from tbl_prepaid_rate where rate_id=%d", rateId)
    
    DbPrepare()
    local result = DbQuery(sqlStr)
    if MSS_ERR_OK~=result or 1~=DbGetRecordCnts() then
        DbEnd()
        return -2
    end
    DbFetchRow()
    rateInfo.interval = DbGetRowIntVal(0)
    rateInfo.tariff = DbGetRowIntVal(1)
    rateInfo.onetimeFee = DbGetRowIntVal(2)
    DbEnd()
    return 1
end


local function db_caller_prepaid_get_sub_balance()
    local sqlStr=string.format("select nBalance from tbl_sub_balance where cSubName='%s'", subNbr)
    
    DbPrepare()
    local result = DbQuery(sqlStr)
    if MSS_ERR_OK~=result or 0==DbGetRecordCnts() then 
        DbEnd()
        return 0 
    end
    DbFetchRow()
    result = DbGetRowIntVal(0)
    DbEnd()
    return result
end


local function db_caller_prepaid_set_sub_balance(new_balance)
    local sqlStr=string.format("update tbl_sub_balance set nBalance=%d where cSubName='%s'", new_balance, subNbr)
    DbPrepare()
    DbQuery(sqlStr)
    DbEnd()
end


local function db_set_cdr()
    if duration==0 then return end
    
    local sqlStr=string.format("insert into tbl_cdr (nDirection, cCaller, cCalled, nSubType, tStartTime, tConnectTime,tReleaseTime,nDuration,nFee) values (0,'%s','%s',1,'%s','%s','%s',%d,%d)",
                                subNbr, fsm.calledNbr, timeToStr(setupTime), timeToStr(answerTime),timeToStr(releaseTime),duration, fee)
    DbPrepare()
    DbQuery(sqlStr)
    DbEnd()
end


local function checkBalance()
    TraceInfo("\t checkBalance")
    if (rateInfo.tariff==0 and rateInfo.onetimeFee==0) or FREE_CALL_IND_YES==fsm.freeCallInd then
        TraceInfo("\t\t current call is free call.")
        return 0
    end

    if balance<=0 then
        TraceInfo("\t\t no enough balance.")
        return -1
    end

    balance = balance - rateInfo.onetimeFee
    allowDuration = ToInt((balance/rateInfo.tariff)*rateInfo.interval)
    TraceInfo("\t\t balance="..balance..",allow duration="..allowDuration)

    if allowDuration>rateInfo.interval then 
        return 1
    else
        TraceInfo("\t\t the balance cannot make a min-interval call.")
        return -2
    end
end

local function sendPA_BalanceInsufficient()
    local paArg = CreatePA()
    paArg.infoSend_annID_annID = 0x07080001
    paArg.infoSend_numOfRepeat = 1
    SendPA(CALLER_LEGID, paArg)
end

-- ---------------------------------------------------------------------------------------
-- 2021-08-10 Only set the 'answerTime' when the call is answered, so detect it directly.
-- 2021-08-16 Fix a stupid typing bug when calculate balance.
-- ---------------------------------------------------------------------------------------
local function calculateFee()
    TraceInfo("\t calculateFee")
    TraceInfo("\t\t setupTime="..setupTime..", answerTime="..answerTime..", releaseTime="..releaseTime)
    if answerTime>0 then
        duration = Duration(releaseTime, answerTime)

        -- refine duration since MSS timer has several seconds intervals
        if expireInd == MSS_YES then
            duration = allowDuration
        end
    else
        duration=0
    end
    TraceInfo("\t\t duration="..duration)
    if duration==0 then return end

    if rateInfo.tariff==0 or FREE_CALL_IND_YES==fsm.freeCallInd then
        TraceInfo("\t\t tariff=0, or free call.")
        fee = 0
    else
        local cnt = ToInt(duration/rateInfo.interval)
        fee = cnt*rateInfo.tariff
        
        -- even it is less than a interval, treat it as an interval
        if duration%rateInfo.interval >0 then
            fee = fee + rateInfo.tariff
        end

        if duration>0 then
            fee = fee + rateInfo.onetimeFee
        end
    end    
    TraceInfo("\t\t fee="..fee)
    
    if fee>0 then
        -- re-get user balance                
        balance = db_caller_prepaid_get_sub_balance()
        balance = balance - fee
        TraceInfo("\t\t balance="..balance)        
        db_caller_prepaid_set_sub_balance(balance)
    end    
end

local function recvAnswer()
    TraceInfo("\t receive answer event")    
    answerTime = Now()
    StartTimer(allowDuration)
    EnterState(ECALLER_PREPAID_WAIT_DISC)
end

local function relCall()
    TraceInfo("\t relCall")
    releaseTime = Now()
    calculateFee()
    EndService()
end

local function balanceExpired()
    TraceInfo("\t balanceExpired")

    expireInd = MSS_YES
    releaseTime = Now()
    
    SendReleaseCall()
    calculateFee()
    EndService()
end

local function relCallForBalanceInsufficient()
    TraceInfo("\t relCallForBalanceInsufficient")
    StopTimer()
    SendReleaseCall(CAUSE_CALLER_BALANCE_NOT_ENOUGH)
    EndService()
end

local function exception()
    TraceInfo("\t exception")
    SendReleaseCall()
    EndService()
end



--*************************************
--
-- Procedures for messages and states
--
--*************************************

-- ---------------------------------------------------------------------------------------
-- 2021-08-10 Since called party can answer the call immediately, such as auto-answer, so
--            it is wrong to set 'answerTime=setupTime' when the service is invoked.
-- ---------------------------------------------------------------------------------------
local function onIdleProc(currMsg)
    TraceInfo("callerPrepaid:onIdleProc")
    setupTime = Now()    
    
    -- get subscriber number
    if MSS_YES==fsm.foSSPInd then
        subNbr = fsm.redirNbr
    else
        subNbr = fsm.callerNbr
    end
    TraceInfo("\t subscriber number is "..subNbr)
    
    -- get rate info
    rateId = db_caller_prepaid_get_rateid()
    TraceInfo("\t prepare rate ID = "..rateId)
    
    local result = db_caller_prepaid_get_rate_info()
    if result <0 then 
        TraceInfo("\t fail to get rate info. result="..result)
        exception() 
    end
    TraceInfo("\t rate info: interval="..rateInfo.interval..",tariff="..rateInfo.tariff..",onetimeFee="..rateInfo.onetimeFee)
    
    -- get subscriber's balance
    balance = db_caller_prepaid_get_sub_balance()
    TraceInfo("\t the balance is "..balance)

    result=checkBalance()
    if result<0 then
        if MSS_YES==fsm.foSSPInd then
            TraceInfo("\t without balance in FO call, just release it.")
            SendReleaseCall(CAUSE_CALLER_BALANCE_NOT_ENOUGH)
            EndService()
            return
        else
            SendCTR(CALLER_LEGID)
            sendPA_BalanceInsufficient()
            StartTimer(60)
            EnterState(ECALLER_PREPAID_WAIT_ANN_END)
            return            
        end

    elseif result==0 then
        SendContinue()
        EndService()
        return 
    end
    
    -- the subscrier has enough balance
    SendRRBE(CALLED_LEGID, DP_O_ANSWER,     DP_MON_NOTIFY)
    SendRRBE(CALLER_LEGID, DP_O_DISCONNECT, DP_MON_NOTIFY)
    SendContinue()

    StartTimer(TIMER_WAIT_ANS)
    EnterState(ECALLER_PREPAID_WAIT_ANS)
end


local function onWaitAnsProc(currMsg)
    TraceInfo("onWaitAnsProc")
    StopTimer()
    if currMsg==ESCP_EVT_ERB then -- it must be answer event
        recvAnswer()
    elseif currMsg==ESCP_EVT_TIMEOUT then
        exception()
    else
        EndService()
    end
end


local function onWaitDiscProc(currMsg)
    TraceInfo("onWaitDiscProc")
    StopTimer()
    
    if currMsg==ESCP_EVT_ERB or currMsg==ESCP_EVT_TRANS_CLOSE then
        relCall() 
    elseif currMsg==ESCP_EVT_TIMEOUT then
        balanceExpired()
    else
        EndService()
    end
end


local function onWaitAnnEndProc(currMsg)
    TraceInfo("onWaitAnnEndProc")

    StopTimer()
    if currMsg==ESCP_EVT_TRANS_CLOSE then
        relCall()
    elseif currMsg==ESCP_EVT_TIMEOUT or currMsg==ESCP_EVT_SRR or currMsg==ESCP_EVT_SRF_ERR then
        relCallForBalanceInsufficient()
    else
        EndService()
    end
end


function MainProc(currMsg)

    if fsm.state==ECALLER_PREPAID_IDLE then
        onIdleProc(currMsg)
    elseif fsm.state==ECALLER_PREPAID_WAIT_ANS then
        onWaitAnsProc(currMsg)
    elseif fsm.state==ECALLER_PREPAID_WAIT_DISC then
        onWaitDiscProc(currMsg)
    elseif fsm.state==ECALLER_PREPAID_WAIT_ANN_END then
        onWaitAnnEndProc(currMsg)
    else
        TraceInfo("\t unknown state: "..fsm.state)
        EndService()
    end    
end


function DestroyProc()
    TraceInfo("DestroyProc")

    releaseTime = Now()
    if answerTime>0 then
        duration = Duration(releaseTime,answerTime)
    end
    db_set_cdr()
end
