--=======================================================================
--miniSIPServer simple conference room service
--
--Copyright (C) MYVOIPAPP,Inc. All Rights Reserved.
--
--Author: Gilson
--
--History:
--2024-09-27 created.
--=======================================================================
require "services"
require "utils.data_string"
require "utils.dba"

--service state
local ECR_STATE_IDLE        = 0
local ECR_STATE_ACTIVE      = 1
local ECR_STATE_P1_WAIT_CC  = 3 -- wait create-conference result
local ECR_STATE_P1_WAIT_P2  = 4 -- wait other members
local ECR_STATE_P1_WAIT_AC  = 5 -- p1 wait attend-conference result
local ECR_STATE_P2_WAIT_AC  = 6 -- other member wait attend-conference result

-- default timer (seconds)
local CR_TIMER_WAIT_RESP = 10
local CR_TIMER_WAIT_P2   = 1800
local CR_TIMER_ACTIVE    = 36000

--====================================
--
-- Cache DB functions
--
--====================================
local function cdb_conf_room_check(roomID, password)
    local sqlStr = string.format("select count(*) from tbl_conf_room where roomID='%s' and (password='%s' or password='')", roomID, password)
    if cdbQuery(sqlStr)==MSS_ERR_OBJ_FOUND and cdbGetRowIntVal(0)==1 then
        return MSS_YES
    else
        return MSS_NO
    end
end

local function cdb_conf_scf_has(confRoomID)
    local sqlStr = string.format("select count(*) from tbl_conf_scf where confRoomID='%s'", confRoomID)
    if cdbQuery(sqlStr)==MSS_ERR_OBJ_FOUND and cdbGetRowIntVal(0)==1 then
        return MSS_YES
    else
        return MSS_NO
    end
end

local function cdb_conf_scf_get_firstScfID(confRoomID)
    local sqlStr = string.format("select firstScfID from tbl_conf_scf where confRoomID='%s'", confRoomID)
    if cdbQuery(sqlStr)==MSS_ERR_OBJ_FOUND then
        return cdbGetRowIntVal(0)
    else
        return MSS_INVALID_OBJ_ID
    end
end

local function cdb_conf_scf_get_confNodeID(confRoomID)
    local sqlStr = string.format("select confNodeID from tbl_conf_scf where confRoomID='%s'", confRoomID)
    if cdbQuery(sqlStr)==MSS_ERR_OBJ_FOUND then
        return cdbGetRowIntVal(0)
    else
        return MSS_INVALID_OBJ_ID
    end
end

local function cdb_conf_scf_del(confRoomID)
    local sqlStr = string.format("delete from tbl_conf_scf where confRoomID='%s' and memCnt=0", confRoomID)
    cdbQuery(sqlStr)
end

local function cdb_conf_scf_add_default(confRoomID, firstScfID)
    local sqlStr = string.format("insert into tbl_conf_scf(confRoomID, firstScfID, TTL) values('%s', %d, %d)",
                                 confRoomID, firstScfID, ssfGetCurrSeconds()+CR_TIMER_ACTIVE)
    if cdbQuery(sqlStr)==MSS_ERR_OK then return MSS_ERR_OK else return MSS_ERR_FAIL end
end

local function cdb_conf_scf_update_confNodeID(confRoomID, confNodeID)
    local sqlStr = string.format("update tbl_conf_scf set confNodeID=%d where confRoomID='%s'", confNodeID, confRoomID)
    if cdbQuery(sqlStr)==MSS_ERR_OK then return MSS_ERR_OK else return MSS_ERR_FAIL end
end

local function cdb_conf_scf_inc_memcnt(confRoomID)
    if confRoomID==nil then return end

    local sqlStr = string.format("update tbl_conf_scf set memCnt=memCnt+1 where confRoomID='%s'", confRoomID)
    if cdbQuery(sqlStr)==MSS_ERR_OK then return MSS_ERR_OK else return MSS_ERR_FAIL end
end

local function cdb_conf_scf_dec_memcnt(confRoomID)
    if confRoomID==nil then return end

    local sqlStr = string.format("update tbl_conf_scf set memCnt=memCnt-1 where confRoomID='%s' and memCnt>0", confRoomID)
    if cdbQuery(sqlStr)==MSS_ERR_OK then return MSS_ERR_OK else return MSS_ERR_FAIL end
end
--====================================
--
-- FSM common functions
--
--====================================
local function initParams()
    fsm.roomID = ""
    fsm.password = ""
    fsm.cldPrefix = currMessage.luaCldPrefix
    TraceInfo("\t called number is '"..fsm.calledNbr.."'")
    TraceInfo("\t service prefix is '"..fsm.cldPrefix.."'")
end

local function getConfInfo()
    TraceInfo("\t getConfInfo")
    local confInfo = StrLstrip(fsm.calledNbr, fsm.cldPrefix)
    local roomID, password = StrLSplitAtStr(confInfo, "*")
    -- not found the '*'
    if roomID==nil then
        if string.len(confInfo)>0 then
            fsm.roomID = confInfo
            return 1
        else
            TraceInfo("\t\t no room ID")
            return -1
        end
    end
    fsm.roomID = roomID
    if password~=nil then fsm.password = password end
    return 2
end

local function checkConfInfo()
    local result = cdb_conf_room_check(fsm.roomID, fsm.password)
    if result==MSS_YES then return 1 else return -1 end
end

local function exceptionProc()
    TraceInfo("\t exceptionProc")
    SendReleaseCall()
    EndService()
end

local function monitorCall()
    SendRRBE(CONTROL_LEGID, DP_O_ABANDON,    DP_MON_INTERUPT)
    SendRRBE(CONTROL_LEGID, DP_O_DISCONNECT, DP_MON_INTERUPT)
end

--====================================
--
-- FSM state functions
--
--====================================
local function onIdle_P1()
    TraceInfo("onIdle_P1")
    if cdb_conf_scf_add_default(fsm.roomID, fsm.scpFsmID)~=MSS_ERR_OK then
        exceptionProc()
        return
    end
    SendCTR(CONTROL_LEGID)
    SendCreateConf(CONTROL_LEGID)

    StartTimer(CR_TIMER_WAIT_RESP)
    EnterState(ECR_STATE_P1_WAIT_CC)
end

local function onIdle_P2()
    TraceInfo("onIdle_P2")
    fsm.confNodeID = cdb_conf_scf_get_confNodeID(fsm.roomID)
    if fsm.confNodeID==MSS_INVALID_OBJ_ID then
        TraceInfo("\t cannot get conf node ID from cache.")
        return exceptionProc()
    end
    SendCTR(CONTROL_LEGID)
    SendAttendConf(CONTROL_LEGID,fsm.confNodeID)
    StartTimer(CR_TIMER_WAIT_RESP)
    EnterState(ECR_STATE_P2_WAIT_AC)
end

local function onIdleProc()
    TraceInfo("conference_room: onIdleProc")
    initParams()

    if getConfInfo()<0 then return exceptionProc() end
    TraceInfo("\t room ID = '"..fsm.roomID.."'")
    TraceInfo("\t password = '"..fsm.password.."'")

    if checkConfInfo()<0 then
        TraceInfo("\t invalid conference room info.")
        return exceptionProc()
    end
    monitorCall()
    if cdb_conf_scf_has(fsm.roomID)==MSS_NO then
        return onIdle_P1()
    else
        return onIdle_P2()
    end
end

local function onP1WaitCCProc(currEvt)
    TraceInfo("onP1WaitCCProc")
    if currEvt==ESCP_EVT_TIMEOUT then
        return exceptionProc()

    elseif currEvt==ESCP_EVT_CREATE_CONF_ACK then
        fsm.confNodeID = currMessage.confNodeID
        TraceInfo("\t confNodeID="..IntToHexStr(fsm.confNodeID))
        cdb_conf_scf_update_confNodeID(fsm.roomID, fsm.confNodeID)
        SendMusicToCaller()
        StartTimer(CR_TIMER_WAIT_P2)
        EnterState(ECR_STATE_P1_WAIT_P2)
    end
end

local function onP1WaitP2Proc(currEvt)
    TraceInfo("onP1WaitP2Proc")
    if currEvt==ESCP_EVT_TIMEOUT then
        return exceptionProc()

    elseif currEvt==ESCP_EVT_SCF_ENTER_CONF_ROOM then
        TraceInfo("\t another member enters the conference room")
        SendAttendConf(CONTROL_LEGID, fsm.confNodeID)
        StartTimer(CR_TIMER_WAIT_RESP)
        EnterState(ECR_STATE_P1_WAIT_AC)
    end
end

local function onP1WaitACAckProc(currEvt)
    TraceInfo("onP1WaitACAckProc")
    if currEvt==ESCP_EVT_TIMEOUT then
        return exceptionProc()

    elseif currEvt==ESCP_EVT_ATTEND_CONF_ACK then
        TraceInfo("\t recv AC_ACK")
        cdb_conf_scf_inc_memcnt(fsm.roomID)
        StartTimer(CR_TIMER_ACTIVE)
        EnterState(ECR_STATE_ACTIVE)
    end
end

local function onP2WaitACAckProc(currEvt)
    TraceInfo("onP2WaitACAckProc")
    if currEvt==ESCP_EVT_TIMEOUT then
        return exceptionProc()

    elseif currEvt==ESCP_EVT_ATTEND_CONF_ACK then
        TraceInfo("\t recv AC_ACK")
        local peerScfID = cdb_conf_scf_get_firstScfID(fsm.roomID)
        SendScfEnterConfRoom(peerScfID)
        cdb_conf_scf_inc_memcnt(fsm.roomID)
        StartTimer(CR_TIMER_ACTIVE)
        EnterState(ECR_STATE_ACTIVE)
    end
end

local function onActiveProc(currEvt)
    TraceInfo("onActiveProc")
    if currEvt==ESCP_EVT_TIMEOUT then
        return exceptionProc()
    end
    -- doesn't care other events
end
--====================================
--
-- Service main function
--
--====================================
function DestroyProc()
    TraceInfo("DestroyProc")
    cdb_conf_scf_dec_memcnt(fsm.roomID)
    cdb_conf_scf_del(fsm.roomID)
end

function MainProc(currEvt)
    if currEvt==ESCP_EVT_TRANS_CLOSE then return EndService() end

    if fsm.state==ECR_STATE_IDLE then
        onIdleProc()
    elseif fsm.state==ECR_STATE_P1_WAIT_CC then
        onP1WaitCCProc(currEvt)
    elseif fsm.state==ECR_STATE_P1_WAIT_P2 then
        onP1WaitP2Proc(currEvt)
    elseif fsm.state==ECR_STATE_P1_WAIT_AC then
        onP1WaitACAckProc(currEvt)
    elseif fsm.state==ECR_STATE_P2_WAIT_AC then
        onP2WaitACAckProc(currEvt)
    elseif fsm.state==ECR_STATE_ACTIVE then
        onActiveProc(currEvt)
    else
        TraceInfo("unknown state: "..fsm.state)
        exceptionProc()
    end
end

-- EOF