--==============================================================================
-- miniSIPServer basic definiations
--
-- Copyright (C) MYVOIPAPP,Inc. All Rights Reserved.
-- 
-- All contents are system definitions, customers SHOULD NOT modify this file!
-- 
-- author: Hong
--==============================================================================

require "utils.data_type"

--#######################################
--
-- Q.850 causes and extended causes
--
--#######################################

-- standard causes
CAUSE_CALL_REJECTED = 0X15
CAUSE_NORMAL_UNSPECIFIED = 0x1F

-- extended causes
CAUSE_CALLER_BALANCE_NOT_ENOUGH = 0x0100|CAUSE_CALL_REJECTED


--#######################################
--
-- SDP definition (ref to 'Esdp_attribute' in MSS CORE)
--
--#######################################
ESDP_ATTR_SENDRECV = 3
ESDP_ATTR_SENDONLY = 4
ESDP_ATTR_RECVONLY = 5
ESDP_ATTR_INACTIVE = 6

-- DP monitor type
DP_MON_INTERUPT = 0
DP_MON_NOTIFY   = 1

-- DPs
DP_ORIG_ATTEMPT_AUTHORIZED = 1
DP_COLLECTED_INFO          = 2
DP_ANALYZED_INFO           = 3
DP_ROUTE_SELECT_FAILURE    = 4
DP_O_BUSY                  = 5
DP_O_NO_ANSWER             = 6
DP_O_ANSWER                = 7
DP_O_MIDCALL               = 8
DP_O_DISCONNECT            = 9
DP_O_ABANDON               = 10
DP_TERM_ATTEMPT_AUTHORIZED = 12
DP_T_BUSY                  = 13
DP_T_NO_ANSWER             = 14
DP_T_ANSWER                = 15
DP_T_MID_CALL              = 16
DP_T_DISCONNECT            = 17
DP_T_ABANDON               = 18
DP_O_TERM_SEIZED           = 19
DP_ORIG_ATTEMPT            = 22
DP_TERM_ATTEMPT            = 23
DP_CALL_ACCEPTED           = 27
DP_O_CALL_PROGRESS         = 28

-- Messages
ESCP_EVT_UNKNOWN            = 0
ESCP_EVT_TIMEOUT            = 1
ESCP_EVT_IDP                = 2
ESCP_EVT_ERB                = 3
ESCP_EVT_TRANS_CLOSE        = 4
ESCP_EVT_ICA_RESP           = 5
ESCP_EVT_CALLBACK_REQ       = 6
ESCP_EVT_SRR                = 7
ESCP_EVT_PNC_RESULT         = 8
ESCP_EVT_SRF_ERR            = 9
ESCP_EVT_SRF_RECORD_ANN_END = 10
ESCP_EVT_ERROR              = 11
ESCP_EVT_LEG_RELD           = 12
ESCP_EVT_CALLBACK_REL       = 13
ESCP_EVT_CREATE_CONF_ACK    = 14
ESCP_EVT_ATTEND_CONF_ACK    = 15
ESCP_EVT_SRF_ANN_END        = 50
ESCP_EVT_SRF_BIND_RESP      = 51
ESCP_EVT_SCF_CALL_RETRIEVE_REQ  = 70
ESCP_EVT_SCF_CALL_RETRIEVE_RESP = 71
ESCP_EVT_SCF_ENTER_CONF_ROOM    = 72
ESCP_EVT_DB_OPERATOR_IDLE       = 91
ESCP_EVT_UA_USER_IDLE = 100

-- LegId (O side)
CALLER_LEGID  = 1
CALLED_LEGID  = 2

-- legID
CONTROL_LEGID = 1
PASSIVE_LEGID = 2

-- Call stage, same with Ecall_stage
ECALL_DIAL  = 0
ECALL_ALERT = 1
ECALL_TALK  = 2

--########################################
--
-- service information indicators
--
--########################################

SERV_INFO_IND_NOT_CARE = 0

-- free call indication
FREE_CALL_IND_NO  = 1
FREE_CALL_IND_YES = 2

-- suppress call level
SERVINFO_SUPPRESS_CALL_LEVEL_NO  = 1
SERVINFO_SUPPRESS_CALL_LEVEL_YES = 2


-- SRF error type, same with Esrf_err_type defined in mss core
ESRF_ERR_OK = 0
ESRF_ERR_INVALID_OBJ = 1
ESRF_ERR_WRONG_SEQUENCE = 2
ESRF_ERR_UNAVAILABLE_RES = 3
ESRF_ERR_FORCE_RELEASED = 4
ESRF_ERR_FIRST_DIGIT_TIMEOUT = 5
ESRF_ERR_IMPROPER_CALLER_RESPONSE = 6
ESRF_ERR_UNKNOWN = 255

-- auto-disconnect indicator
AUTO_DISCONNECT_ALLOWED     = 0
AUTO_DISCONNECT_NOT_ALLOWED = 1

ESRF_INFO_ANN_ID = 0
ESRF_INFO_TEXT   = 1

ESRF_INFO_VAR_NULL  = 0
ESRF_INFO_VAR_PRICE = 1
ESRF_INFO_VAR_DIGIT = 2
ESRF_INFO_VAR_MULTI_ID = 3
ESRF_INFO_VAR_DURATION = 4
ESRF_INFO_VAR_ANN_FILE = 5

ESRF_SRR_ANN_END    = 0
ESRF_SRR_ANN_BEGIN  = 1

ERECORD_ANN_TYPE_VM = 0
ERECORD_ANN_TYPE_GREETING = 1

INTANN_PA_TO_COL_DIG = 0
INTANN_STOP_FOR_FIRST_DIG = 1
INTANN_NO_INTERRUPT = 2

-- common announcement id 
ANN_ID_COMMON_MUSIC     = 0x00080002
ANN_ID_COMMON_RINGBACK  = 0x00080003
ANN_ID_OP_SUCCESS       = 0X01080001



function CreateConnectArg()
    local connArg={}

    connArg.destNbr = ""
    connArg.callerNbrInd = MSS_NO
    connArg.callerNbr = ""
    connArg.retrieveProfileInd = MSS_NO
    connArg.dialPlanInd = MSS_NO
    connArg.dialPlan = ""
    -- service information indicators
    connArg.servInfoInd = MSS_NO
    connArg.freeCallInd = MSS_NO
    connArg.suppressCallLevel =  MSS_NO
    return connArg
end

function CreatePA()
    local paArg={}
    paArg.discForbid = AUTO_DISCONNECT_NOT_ALLOWED
    paArg.notifyEnd = MSS_YES
    paArg.notifyBegin = MSS_NO

    paArg.infoSend_annID_annType = ESRF_INFO_ANN_ID
    paArg.infoSend_audioOwner = ""
    paArg.infoSend_annID_annID = 0
    paArg.infoSend_duration = 0
    paArg.infoSend_numOfRepeat = 0
    
    paArg.infoSend_varType = 0
    paArg.infoSend_var_price = 0
    paArg.infoSend_var_digit = 0
    paArg.infoSend_var_duration = 0
    paArg.infoSend_var_annID = 0
    paArg.infoSend_var_annFileName = ""

    paArg.infoSend_varType2 = 0
    paArg.infoSend_var_price2 = 0
    paArg.infoSend_var_digit2 = 0
    paArg.infoSend_var_duration2 = 0
    paArg.infoSend_var_annID2 = 0
    paArg.infoSend_var_annFileName2 = ""
    return paArg
end


function CreatePNC()
    local pncArg={}

    pncArg.infoSend_annID_annType = ESRF_INFO_ANN_ID
    pncArg.infoSend_audioOwner = ""
    pncArg.infoSend_annID_annID = 0
    pncArg.infoSend_duration = 0
    pncArg.infoSend_numOfRepeat = 0

    pncArg.infoSend_varType = 0
    pncArg.infoSend_var_price = 0
    pncArg.infoSend_var_digit = 0
    pncArg.infoSend_var_duration = 0
    pncArg.infoSend_var_annID = 0
    pncArg.infoSend_var_annFileName = ""

    pncArg.infoSend_varType2 = 0
    pncArg.infoSend_var_price2 = 0
    pncArg.infoSend_var_digit2 = 0
    pncArg.infoSend_var_duration2 = 0
    pncArg.infoSend_var_annID2 = 0
    pncArg.infoSend_var_annFileName2 = ""

    pncArg.minNumOfDigits = 1
    pncArg.maxNumOfDigits = 30
    pncArg.firstDigitTimeout = 60
    pncArg.intervalDigitTimeout = 20
    pncArg.interuptAnnInd = INTANN_STOP_FOR_FIRST_DIG
    pncArg.endDigit="#"
    pncArg.cancelDigit="*"

    pncArg.discForbid = AUTO_DISCONNECT_NOT_ALLOWED
    return pncArg
end


function CreateRecordANN()
    local rdAnnArg={}
    rdAnnArg.type = ERECORD_ANN_TYPE_VM
    rdAnnArg.callerNbr = ""
    rdAnnArg.subNbr = ""
    return rdAnnArg
end


function CreateICA()
    local icaArg={}
    icaArg.calledNbr = ""

    icaArg.callerNbrInd = 0
    icaArg.callerNbr = ""

    icaArg.callerSrfInfoInd = 0
    icaArg.callerSrfID = 0
    return icaArg
end

function CreateBridge2Parties()
    local b2pArg={}
    b2pArg.peerSrfID = 0
    b2pArg.peerSsfFsmID = 0
    return b2pArg
end

-- EOF
